// C Source File
// Created 2/3/01; 8:54:24 PM

// Serial Driver program for demonstrating the use of functions that allow you
// to communicate at standard baud rates through your calculators link port.  The
// "tip" link port connector ( red wire ) is used as the Receive line, and the
// "middle" link port connector ( white wire ) is used as the Send line.

// AUTHOR: Josh Christensen
//   DATE: 2/25/01
//  EMAIL: joshc@ieee.org

#define OPTIMIZE_ROM_CALLS    // Use ROM Call Optimization

#define SAVE_SCREEN           // Save/Restore LCD Contents

#define LINK_PORT 		0x60000E
#define LINK_STATUS 	0x60000C
#define CLK_L_DAT_L 	0x0300
#define CLK_H_DAT_L 	0x0100
#define CLK_L_DAT_H 	0x0200
#define CLK_H_DAT_H 	0x0000
#define READ_CLOCK		0x0800
#define READ_DATA			0x0400
#define K_ESC					0x108
#define K_CLEAR				0x107
#define K_CAT					0x116
#define K_APPS				0x109
#define K_BACKSPACE		0x101
#define K_UP					0x151
#define K_RIGHT				0x158
#define K_DOWN				0x154
#define K_LEFT				0x152
#define K_F1					0x10C
#define K_F2					0x10D
#define K_F3					0x10E
#define K_F4					0x10F
#define K_F5					0x110
#define K_ENTER				0xD
#define K_X						0x78
#define K_Y						0x79
#define K_Z						0x7A
#define K_T						0x74
#define K_PLUS				0x2B
#define K_MINUS				0x2D
#define LF						0x0A
#define CR						0x0D
#define BS						0x08
#define FF						0x0C
#define MAX_SERIAL_DATA 	 100		// 100 bytes		
#define MAX_TERMINAL_INPUT 100

#include <tigcclib.h>

/*
#include <graph.h>            // Support Graphics
#include <kbd.h>
#include <stdio.h>
#include <stdlib.h>
#include <system.h>
#include <string.h>
#include <timath.h>
#include <tigcc
*/

short _ti89;                  // Produce .89Z File
short _ti92plus;              // Produce .9XZ File

volatile int *pLinkPort = (int*) LINK_PORT;
int *pLinkStatus = (int*) LINK_STATUS;
int iOldPortStatus = 0;
unsigned BitPeriod1 = 0;
unsigned BitPeriod1Tx = 0;
unsigned BitPeriod15 = 0;
unsigned BytesToGet = 5;
unsigned BaudSenseBreakPoints[ 9 ] = { 1659, 555, 164, 81, 39, 18, 7, 3, 0 };
unsigned BaudRateIndex = 4;
unsigned BaudRateBreakPoints[ 9 ] = { 110, 300, 1200, 2400, 4800, 9600, 19200, 38400, 57600 }; 
unsigned BaudSense = 0;
unsigned RxBytesReceived = 0;
long unsigned Timeout = 0x0003FFFF;
unsigned char SerialDataRx[ MAX_SERIAL_DATA ] = "0"; 
unsigned char SerialDataTx[ MAX_SERIAL_DATA ] = "0";
unsigned char TerminalInput[ MAX_TERMINAL_INPUT ] = "0";
char Buffer1[ 40 ] = "0";
char BlankLine[] = "                           ";
enum allowesc { INHIBIT_ESC = 0, ALLOW_ESC = 1 } ;  
enum allowesc eAllowEscGlobal = INHIBIT_ESC;

void OpenLinkPort ();
void CloseLinkPort();
void DrawParameters();
void DoDelay( unsigned DelayTime );
void DoScreenDump();
void SendSerialData( unsigned BytesToSend );
void GetBaudSelection();
void SetBaudRate( unsigned BaudRateIndexToSet );
void DoTiTerminalHost();
void GetTerminalInput();
int SenseBaudRate( unsigned PeriodsToCompare );
int GetSerialData( unsigned NumberOfBytes, long unsigned TimeOutValue, enum allowesc eAllowEsc );
float RoundFloat( float ValueToRound );

// Main Function
void _main(void)
{
	int KeyIn = 0;
	
	OpenLinkPort();

	*pLinkPort = CLK_L_DAT_H;					// Setting Tx (white) low, inactive and
																		// setting Rx (red) line high so we can sense its state.
	SetBaudRate( BaudRateIndex );
	
	ClrScr();
	
	DrawStr( 30, 0, "Serial Driver ", A_NORMAL );

	DrawParameters();
	
	while ( KeyIn != K_ESC )
	{
		if ( kbhit() )
		{
			KeyIn = ngetchx();
		
			switch ( KeyIn )
			{	
				case K_UP:
					++BitPeriod1;
					break;
				case K_DOWN:
					--BitPeriod1;
					break;
				case K_RIGHT:
					++BitPeriod15;
					break;
				case K_LEFT:
					--BitPeriod15;
					break;
				case K_T:
					BitPeriod1 += 30;
					break;
				case K_Z:
					BitPeriod1 -= 30;
					break;
				case K_Y:
					BitPeriod15 += 30;
					break;
				case K_X:
					BitPeriod15 -= 30;
					break;
				case K_F1:
					sprintf( SerialDataTx, "\n\rHey, this is your TI-89 talking to you at %u baud.", BaudRateBreakPoints[ BaudRateIndex ] );
					SendSerialData( 0xFFFF );
				  DrawStr( 30, 0, "Message Sent  ", A_REPLACE );
				  break;
				case K_F5:
					++BytesToGet;
					break;
				case K_F4:
					--BytesToGet;
					break;
				case K_F2:
					Timeout ^= 0x0003FFFF;
					break;
				case K_F3:
					(int) eAllowEscGlobal ^= 0x0001;
					break;
				case K_CAT:
					DoTiTerminalHost();
					ClrScr();
					DrawStr( 30, 0, "Done Hosting  ", A_REPLACE );
					break;
				case K_APPS:
					GetBaudSelection();
					ClrScr();
					DrawStr( 30, 0, "Baud Selected ", A_REPLACE );
					break;
				case K_MINUS:
					DoScreenDump();
					ClrScr();
					DrawStr( 30, 0, "Done Dumping  ", A_REPLACE);
					break;
				case K_PLUS:
					DrawStr( 30, 0, "Sensing Baud  ", A_REPLACE);
					if ( SenseBaudRate( 0x000A ) ) 
					{
						DrawStr( 30, 0, "Baud Not Found", A_REPLACE);
					}
					else
					{
						DrawStr( 30, 0, "Done Sensing  ", A_REPLACE);
					}
					break;
				case K_ENTER:
					DrawStr( 30, 0, "Acquiring Data", A_REPLACE);
					if ( GetSerialData( BytesToGet, Timeout, eAllowEscGlobal ) )
					{
						DrawStr( 30, 0, "Timeout       ", A_REPLACE);	
					}
					else
					{
						DrawStr( 30, 0, "Data Acquired ", A_REPLACE);
					}
					
					DrawStr( 0, 78, BlankLine, A_REPLACE );
					DrawStr( 0, 78, SerialDataRx, A_REPLACE );
					sprintf( Buffer1, "Byte 1 = 0x%2X", SerialDataRx[ 0 ] );
					DrawStr( 0, 89, Buffer1, A_REPLACE );
					break;
			}
			
			DrawParameters();
			
		}

	}
	CloseLinkPort();
}


void DrawParameters()
{
	sprintf( Buffer1, "1.5 Cycle Delay = %5u", BitPeriod15 );
	DrawStr( 0, 15, Buffer1, A_REPLACE );
	sprintf( Buffer1, "1 Cycle Delay   = %5u", BitPeriod1 );
	DrawStr( 0, 24, Buffer1, A_REPLACE );
	sprintf( Buffer1, "Bytes To Get    = %5u", BytesToGet );
	DrawStr( 0, 33, Buffer1, A_REPLACE );
	sprintf( Buffer1, "Raw Baud Sense  = %5u", BaudSense );
	DrawStr( 0, 42, Buffer1, A_REPLACE );
	sprintf( Buffer1, "Baud Rate       = %5u", BaudRateBreakPoints[ BaudRateIndex ] );
	DrawStr( 0, 51, Buffer1, A_REPLACE );

	if ( Timeout == 0 )
	{
		DrawStr( 0, 65, "Timeout=OFF", A_REPLACE );	
	}
	else
	{
		DrawStr( 0, 65, "Timeout=ON ", A_REPLACE);
	}

	if ( eAllowEscGlobal == INHIBIT_ESC )
	{
		DrawStr( 80, 65, "CheckESC=OFF", A_REPLACE);
	}
	else
	{
		DrawStr( 80, 65, "CheckESC=ON ", A_REPLACE);
	}
	
	return;
}


void GetBaudSelection()
{
	int KeyIn = 0;
	unsigned CurrentBaudIndex = BaudRateIndex;
	
	ClrScr();
	DrawStr( 20, 0, "Select Baud Rate", A_NORMAL );

	sprintf( Buffer1, "Baud Rate = %5u", BaudRateBreakPoints[ CurrentBaudIndex ] );
	DrawStr( 20, 20, Buffer1, A_REPLACE );

	while ( KeyIn != K_ESC )
	{
		if ( kbhit() )
		{
			KeyIn = ngetchx();
			
			switch ( KeyIn )
			{				
				case K_RIGHT:
					if ( CurrentBaudIndex < 8 )
					{
						++CurrentBaudIndex;
					}
					break;
					
				case K_LEFT:
					if ( CurrentBaudIndex > 0 )
					{
						--CurrentBaudIndex;
					}
					break;
			
				case K_ENTER:
					SetBaudRate( CurrentBaudIndex );	
					KeyIn = K_ESC;
					break;
			}
			
			sprintf( Buffer1, "Baud Rate = %5u", BaudRateBreakPoints[ CurrentBaudIndex ] );
			DrawStr( 20, 20, Buffer1, A_REPLACE );
	
		}
	}
	return;
}


void DoScreenDump()
{
	clrscr();
	
	while ( (_rowread( 0xFFBF ) & 0x00FF ) == 0 )
	{
		if ( GetSerialData( 1, 0x00000FFF, INHIBIT_ESC ) == 0 )	
		{
			putchar( SerialDataRx[ 0 ] );
		}
	}
	ngetchx();
	return ;
}


void DoTiTerminalHost()
{
	unsigned *StartAddress, *StopAddress, *Address;
	unsigned char AllNulls[ 8 ] = { 0, 0, 0, 0, 0, 0, 0, 0 };
	
	ClrScr();
	DrawStr( 13, 0, "Terminal Host Running", A_REPLACE );

	SerialDataTx[ 0 ] = FF;
	SendSerialData( 1 );
	strcpy( SerialDataTx, "TI-89 Digital Unix\n\n\r" ) ;
	SendSerialData( 0xFFFF );
	strcpy( SerialDataTx, "Ver 0.12, 2-24-01\n\n\r" );
	SendSerialData( 0xFFFF );
	strcpy( SerialDataTx, "Login: " );
	SendSerialData( 0xFFFF );
	GetTerminalInput();
	sprintf( Buffer1, "%s Has Logged In", TerminalInput );
	DrawStr( 10, 30, Buffer1, A_REPLACE );
	sprintf( SerialDataTx, "\n\n\rWelcome %s, Root Access Granted", TerminalInput );
	SendSerialData( 0xFFFF );

	while ( 1 )
	{
		strcpy( SerialDataTx, "\n\n\r>" );
		SendSerialData( 0xFFFF );
		GetTerminalInput();
		
		strcpy( SerialDataTx, "\n\n\r" );
		SendSerialData( 0xFFFF );

		if ( strcmp( TerminalInput, "mem" ) == 0 )
		{
			sprintf( SerialDataTx, "%6u Bytes Free", HeapAvail() );
			SendSerialData( 0xFFFF );
		}
		else if ( strcmp( TerminalInput, "handles" ) == 0 )
		{
			sprintf( SerialDataTx, "%6u Handles Free", FreeHandles() );
			SendSerialData( 0xFFFF );	
		}
		else if ( strcmp( TerminalInput, "baud" ) == 0 )
		{
			sprintf( SerialDataTx, "Current Baud Rate = %6u", BaudRateBreakPoints[ BaudRateIndex ] );
			SendSerialData( 0xFFFF );	
		}
		else if ( strcmp( TerminalInput, "serial" ) == 0 )
		{
			AB_serno( TerminalInput ); 
			sprintf( SerialDataTx, "Serial Number: %s", TerminalInput );
			SendSerialData( 0xFFFF );	
		}
		else if ( strcmp( TerminalInput, "dump" ) == 0 )
		{
			strcpy( SerialDataTx, "Start Address: " );
			SendSerialData( 0xFFFF );		
			GetTerminalInput();
			StartAddress = (unsigned*) atol( TerminalInput );
			strcpy( SerialDataTx, "\n\rStop Address : " );
			SendSerialData( 0xFFFF );		
			GetTerminalInput();
			StopAddress = (unsigned*) atol( TerminalInput );
			strcpy( SerialDataTx, "\n\n\r" );
			SendSerialData( 0xFFFF );		

			if ( ( (long unsigned) StartAddress ) & 0x0001 )
			{
				strcpy( SerialDataTx, "\n\n\rError! Start Address Must Be Even" );
				SendSerialData( 0xFFFF );			
			}
			else
			{
				for ( Address = StartAddress; Address <= StopAddress; Address++ )
				{
					sprintf(SerialDataTx, "%04X ", (*Address) );
					SendSerialData( 0xFFFF );
				}
			}
		
		}
		else if ( strcmp( TerminalInput, "help" ) == 0 )
		{
			strcpy( SerialDataTx, "Valid Commands:" );
			SendSerialData( 0xFFFF );	
			strcpy( SerialDataTx, "\n\n\rmem\n\rhandles\n\rbaud\n\rserial\n\rdump\n\rexit\n\rhelp" );
			SendSerialData( 0xFFFF );	
		}
		else if ( strcmp( TerminalInput, "exit" ) == 0 )
		{
			strcpy( SerialDataTx, "Logging Out" );
			SendSerialData( 0xFFFF );		
			break;
		}		
		else if ( strncmp( TerminalInput, AllNulls, 8 ) == 0 )		// The case of calc and computer getting
		{																													// disconnected, exit gracefully.
			strcpy( SerialDataTx, "Logging Out" );
			SendSerialData( 0xFFFF );		
			break;
		}		
		else
		{
			strcpy( SerialDataTx, "Bad Command Or File Name" );
			SendSerialData( 0xFFFF );		
		}

	}

	strcpy( SerialDataTx, "\n\n\r" );
	SendSerialData( 0xFFFF );

	return;
}


void GetTerminalInput()
{
	unsigned char TermKey;
	unsigned InputIndex = 0;

	while ( 1 )
	{
		GetSerialData( 1, 0, INHIBIT_ESC );
		TermKey = SerialDataRx[ 0 ];
		if ( TermKey == K_ENTER )
		{
			break;
		}
		else if ( ( TermKey == BS ) && ( InputIndex > 0 ) )
		{
			--InputIndex;	
		}
		else
		{
			TerminalInput[ InputIndex ] = TermKey;
			++InputIndex;	
			if ( InputIndex >= ( MAX_TERMINAL_INPUT - 1 ) )
			{
				break;
			}
		}
		
		SerialDataTx[ 0 ] = TermKey;
		SendSerialData( 1 );
	} 	
	
	TerminalInput[ InputIndex ] = 0;
	return;
}


//---------------------------------------------------------------------------------------
//		 Function: 	GetSerialData()
//
//	Description: 	Monitor data line, reading in serial bytes as they are detected.
//								This function will return when the number of bytes specified has
//								been received, or when the line has been idle longer than the
//								time limit (if enabled), or when ESC is pressed (if enabled).
//
//	 Parameters:	unsigned NumberOfBytes
//									This is the number of bytes that need to be recieved to return.
//
//								long unsigned TimeOutValue
//									If there is no activity on the data line for this relative amount
//									of time, this function will return.  Set to 0x00000000 to
//									inhibit the timeout checks.
//
//								enum allowesc eAllowEsc
//									Parameter that will decide if the ESC key will be checked.
//									If eAllowEsc == ALLOW_ESC, a press of the ESC key will cause this 
//									function to return.
//									
//			Returns:	0 ->  The number of bytes requested has been found, and are stored
//											in SerialDataRx[] with null termination.
//								1 ->  The idle time exceeded the TimeOutValue, or ESC was pressed	
//											SerialDataRx[] will contain bytes recieved before the timeout
//											or ESC happened, and is always null terminated.  The number of
//											bytes received before timeout/ESC is stored in variable RxBytesReceived.
//---------------------------------------------------------------------------------------
//
int GetSerialData( unsigned NumberOfBytes, long unsigned TimeOutValue, enum allowesc eAllowEsc )
{
	unsigned BitCycle, Index = 0;
	long unsigned IdleCounter;
	unsigned char ByteIn = 0;

	IdleCounter = TimeOutValue;

	OSSetSR (0x0700);														// Disable all interrupts, so our critical 
																							// timing is pure and not affected
	*pLinkPort = CLK_L_DAT_H;
	
	if ( eAllowEsc == ALLOW_ESC )
	{
	
		while ( 1 )
		{
			if (_rowread( 0xFFBF ) & 0x00FF ) 
			{
				ngetchx();
				break;
			}
	
			if ( TimeOutValue )
			{
				if ( --IdleCounter == 0 )
				{
					break;
				}
			}
		
			if ((*pLinkPort & READ_DATA) == 0)		// Serial port going high (+V) (Start Bit)	
			{
				DoDelay( BitPeriod15 ) ;
										
				for (BitCycle = 0; BitCycle < 8; BitCycle++)
				{
					if ( *pLinkPort & READ_DATA )
					{
						ByteIn |= (1 << BitCycle);
					}
					DoDelay( BitPeriod1 );
				}
			
				SerialDataRx[ Index ] = ByteIn ;
				++Index ;
				
				if ( Index >= NumberOfBytes )
				{
					SerialDataRx[ Index ] = 0;
					RxBytesReceived = Index;
					OSSetSR (0x0000);
					return 0;
				}
	
				ByteIn = 0;
				IdleCounter = TimeOutValue;
	
			}	// End if ( Serial Active )
		
		} // End of while ( 1 )

	}
	else // if ( eAllowEsc == ALLOW_ESC )
	{
	
		while ( 1 )
		{
			if ( TimeOutValue )
			{
				if ( --IdleCounter == 0 )
				{
					break;
				}
			}
		
			if ((*pLinkPort & READ_DATA) == 0)		// Serial port going high (+V) (Start Bit)	
			{
				DoDelay( BitPeriod15 ) ;
										
				for (BitCycle = 0; BitCycle < 8; BitCycle++)
				{
					if ( *pLinkPort & READ_DATA )
					{
						ByteIn |= (1 << BitCycle);
					}
					DoDelay( BitPeriod1 );
				}
			
				SerialDataRx[ Index ] = ByteIn ;
				++Index ;
				
				if ( Index >= NumberOfBytes )
				{
					SerialDataRx[ Index ] = 0;
					RxBytesReceived = Index;
					OSSetSR (0x0000);
					return 0;
				}
	
				ByteIn = 0;
				IdleCounter = TimeOutValue;
	
			}	// End if ( Serial Active )
		
		} // End of while ( 1 )
		
	} // End if ( eAllowEsc == ALLOW_ESC )

	SerialDataRx[ Index ] = 0;
	RxBytesReceived = Index;
	OSSetSR (0x0000);
	return 1;

}	// End of GetSerialData()


//---------------------------------------------------------------------------------------
//		 Function: 	SendSerialData()
//
//	Description:  This function will transmit the number of bytes specified of 
//								SerialDataTx[] at the current baud rate.  The data is transmitted
//								on the clock line (white wire) at 8 data bits, no parity.
//								
//	 Parameters:	unsigned BytesToSend
//									This is the number of bytes (characters) that are to be transmitted out
//									of SerialDataTx[].  Set to 0xFFFF to transmit characters until the
//									null termination is found.
//
//			Returns:	None.
//---------------------------------------------------------------------------------------
//
void SendSerialData( unsigned BytesToSend )
{
	unsigned ByteIndex, BitCycle;
	unsigned char CurrentByte;

	OSSetSR (0x0700);														// Disable all interrupts, so our critical 
	  																					// timing is pure and not affected
	*pLinkPort = CLK_L_DAT_H;

	for ( ByteIndex = 0; ByteIndex < BytesToSend; ByteIndex++ )
	{
		CurrentByte = SerialDataTx[ ByteIndex ];

		if ( BytesToSend == 0xFFFF ) 
		{
			if ( CurrentByte == 0 )
			{
				break;
			}
		}

		*pLinkPort = CLK_H_DAT_H;									// Send Start Bit
						
		DoDelay( BitPeriod1Tx );
		
		for (BitCycle = 0; BitCycle < 8; BitCycle++)
		{
			if ( CurrentByte & ( 1 << BitCycle ) )
			{
				*pLinkPort = CLK_L_DAT_H;
			}
			else
			{
				*pLinkPort = CLK_H_DAT_H;
			}
			DoDelay( BitPeriod1Tx );
		}
		
		*pLinkPort = CLK_L_DAT_H;
		DoDelay( BitPeriod1Tx );									// Send Stop Bit
	}
		
	OSSetSR (0x0000);
	return;
	
}


void DoDelay( unsigned DelayTime )
{
	int Temp;
	
	while ( DelayTime-- )
	{
		Temp = *pLinkStatus;
	}

	return;	
}


//---------------------------------------------------------------------------------------
//		 Function: 	SenseBaudRate()
//
//	Description: 	Monitor the data line and as it makes transitions
//								(assuming that something is connected to the calc 
//								and talking at some baud rate) try to find the shortest
//								pulse width, monitoring both positive and negative pulses.
//
//	 Parameters:	unsigned PeriodsToCompare
//									This is a relative time to wait while monitoring data line
//									before returning.  A single value passed to this function will
//									take longer to return if there is no data line activity.
//
//			Returns:	0 ->  Some activity was detected and this was used to calculate
//											new timing information.
//								1 ->  No valid activity was detected, no timing information updated.	
//---------------------------------------------------------------------------------------
//
int SenseBaudRate( unsigned PeriodsToCompare )
{
	unsigned ActiveTime, InactiveTime, BaudIndex, BitTimeMin = 0xFFFF;
	float fBitPeriod1, fBitPeriod15, fBaudSense;
	
	OSSetSR (0x0700);														// Disable all interrupts, so our critical 
																							// timing is pure and not affected
	*pLinkPort = CLK_L_DAT_H;	

	while ( --PeriodsToCompare > 0 )
	{
	
		if ( (*pLinkPort & READ_DATA) == 0 )			// Serial port going high (+V) (Start Bit)
		{
			ActiveTime = 0;
			while ( (*pLinkPort & READ_DATA) == 0 )			
			{
				++ActiveTime;
				if ( ActiveTime == 0xFFFF )
				{
					break;
				}
			}
	
			if ( ActiveTime < BitTimeMin )
			{
				BitTimeMin = ActiveTime;
			}
		}
		

		if ( *pLinkPort & READ_DATA )							// Serial port going low (-V)
		{
			InactiveTime = 0;
			while ( *pLinkPort & READ_DATA )			
			{
				++InactiveTime;
				if ( InactiveTime == 0xFFFF )
				{
					break;
				}
			}
	
			if ( InactiveTime < BitTimeMin )
			{
				BitTimeMin = InactiveTime;
			}
		}
	}
		
	if ( BitTimeMin == 0xFFFF )		// No valid sense has occurred
	{
		OSSetSR (0x0000);						// Enable Interrupts
		return 1;		
	}

	BaudSense = BitTimeMin;
	fBaudSense = (float) BitTimeMin;
	
	for ( BaudIndex = 0; BaudIndex <= 8; BaudIndex++ )
	{
		if ( BaudSense > BaudSenseBreakPoints[ BaudIndex ] )
		{
			break;
		} 
	}
	
	BaudRateIndex = BaudIndex;
	
	// Use a linear interpolation on Bit Periods based off of BaudSense
	//
	fBitPeriod1 = ( fBaudSense * 4.8051 ) - 8.7589;
	fBitPeriod15 = ( fBaudSense * 7.2 ) - 6.8;
	
	BitPeriod1 = (int) RoundFloat( fBitPeriod1 );
	BitPeriod15 = (int) RoundFloat( fBitPeriod15 );
	
	switch ( BaudRateBreakPoints[ BaudRateIndex ] )
	{
		// Patch in the bit periods for the fastest speeds.  They cannot be calculated
		// accurately because BaudSense has little resolution at these speeds.
		// These speeds will not "self correct" to battery voltage and clock offset.

		case 57600:
			BitPeriod1 = 2;
			BitPeriod15 = 10;
			BitPeriod1Tx = 0;
			break;
			
		case 38400:
			BitPeriod1 = 13;
			BitPeriod15 = 27;
			BitPeriod1Tx = 11;
			break;
			
		case 19200:
			BitPeriod1Tx = 44;
			break;
			
		default:
			BitPeriod1Tx = BitPeriod1;	
			break;	
	}

	OSSetSR (0x0000);													// Enable Interrupts
	return 0;	
}


void SetBaudRate( unsigned BaudRateIndexToSet )
{
	BaudRateIndex = BaudRateIndexToSet;
	
	switch ( BaudRateIndexToSet )
	{
		case 0: // 110
			BitPeriod1 = 11658;
			BitPeriod15 = 17475;
			BitPeriod1Tx = BitPeriod1;
			break;
			
		case 1:	// 300
			BitPeriod1 = 4259;
			BitPeriod15 = 6397;
			BitPeriod1Tx = BitPeriod1;
			break;
			
		case 2:	// 1200
			BitPeriod1 = 1048;
			BitPeriod15 = 1577;
			BitPeriod1Tx = BitPeriod1;
			break;
	
		case 3: // 2400
			BitPeriod1 = 515;
			BitPeriod15 = 778;
			BitPeriod1Tx = BitPeriod1;
			break;
			
		case 4: // 4800
			BitPeriod1 = 247;
			BitPeriod15 = 362;
			BitPeriod1Tx = BitPeriod1;
			break;
			
		case 5: // 9600
			BitPeriod1 = 113;
			BitPeriod15 = 162;
			BitPeriod1Tx = BitPeriod1;
			break;
			
		case 6: // 19200
			BitPeriod1 = 46;
			BitPeriod15 = 76;
			BitPeriod1Tx = 44;
			break;
			
		case 7: // 38400
			BitPeriod1 = 13;
			BitPeriod15 = 27;
			BitPeriod1Tx = 11;
			break;
			
		case 8: // 57600
			BitPeriod1 = 2;
			BitPeriod15 = 10;
			BitPeriod1Tx = 0;
			break;

	}
	return;
}


void OpenLinkPort ()
{
	iOldPortStatus = *pLinkStatus;
	*pLinkStatus = 0xFF00;
	return;
}


void CloseLinkPort()
{
	*pLinkPort = 0x0000;
	*pLinkStatus = iOldPortStatus;
	return;
}


float RoundFloat( float ValueToRound )
{
	float Frac, RoundedValue, *pTempCrap = &RoundedValue;
	
	Frac = modf( ValueToRound, pTempCrap );

	if ( Frac >= 0 )
	{
		if ( Frac > 0.5 )
		{
			RoundedValue = ceil( ValueToRound );
		}
		else
		{
			RoundedValue = floor( ValueToRound );
		}
	}
	else
	{
		if ( Frac < -0.5 )
		{
			RoundedValue = floor( ValueToRound );
		}
		else
		{
			RoundedValue = ceil( ValueToRound );
		}
	}

	return ( RoundedValue );

}













